<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\File\File;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIS\Controllers\TeknisaCrud;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Response;
use Teknisa\Libs\Service\CRUD;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\FileUploader;
use Teknisa\FIS\Modules\Acessorias\Services\Fis75001ParametrosEfdReinfService;

class Fis75001ParametrosEfdReinfController extends TeknisaCrud
{
    protected $Fis75001ParametrosEfdReinf;
    protected $dataSourceName = 'paramempreinf';
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $fileUploader;
    protected $fileServerBaseUrl;
    protected $connection;
    protected $crudService;
    protected $Fis75001ParametrosEfdReinfService;

    const DEV_LINK  = 'http://midia.teknisa.com/files/';
    const PROD_LINK = 'https://midia.teknisa.com/files/';
    const EXTENSION_PFX = 'pfx';
    const EXTENSION_P12 = 'p12';
    const EXTENSION_PNG = 'png';
    const EXTENSION_JPG = 'jpg';

    public function __construct(\Teknisa\Libs\Service\CRUD $crudService, ParameterBag $parameterBag, Environment $environment, EntityManager $entityManager, newCode $newCode, $fileUploader, $fileServerBaseUrl, Fis75001ParametrosEfdReinfService $Fis75001ParametrosEfdReinfService)
    {
        parent::__construct($crudService, $parameterBag);
        $this->environment = $environment;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->fileUploader = $fileUploader;
        $this->connection = $entityManager->getConnection();
        $this->Fis75001ParametrosEfdReinfService = $Fis75001ParametrosEfdReinfService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $newResponse = $response;
            $this->dataSourceName = 'paramempreinf_classtribut_represlegal';
            parent::find($request, $response);
            $dataSets = $response->getDataSets();
            if(isset($dataSets[0])) {
                $rows = $dataSets[0]->getRows();
                if($rows) {
                    foreach($rows as &$row) {
                        $objB64 = $this->montaB64Certificado($row['DSDIRCERTIFICADO']);
                        $row['CERTIFICADONAME']   = $objB64['name'];
                        $row['DSDIRCERTIFICADO'] = [$objB64];
                    }
                }
            }
            $dataSet = new DataSet('paramempreinf', $rows);
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function save(DTO\Request $request, DTO\Response $response){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $rows = $this->crudService->getRowsToSave($request, 'paramempreinf');

            $this->dataSourceName = 'paramempreinf';
            if(isset($rows[0]['DSDIRCERTIFICADO'][0])) {
                $file = $rows[0]['DSDIRCERTIFICADO'][0];
                $dsDirCert = '<'.$file['name'].'>'.$file['path'];
                $rows[0]['DSDIRCERTIFICADO'] = $dsDirCert;
            }
            else {
                $rows[0]['DSDIRCERTIFICADO'] = '';
            }
            if(isset($rows[0]['CDSENHACERTDIG']) && ($rows[0]['senhaAlterada'] === true || $rows[0]['__is_new'] === true)) {
                //$rows[0]['CDSENHACERTDIG'] = $this->encrypt($rows[0]['CDSENHACERTDIG']);
                $rows[0]['CDSENHACERTDIG'] = Utilities::zeedhiEncrypt($rows[0]['CDSENHACERTDIG']);
            }
            $this->crudService->save($this->dataSourceName, array($rows[0]));
            $this->entityManager->flush();

            if (!isset($rows[1]))  {
                $rows[1] = $rows[0];
                $rows[1]['DTFINHISTORICO'] = null;
                $rows[1]['DTINIHISTORICO'] = null;
            }

            $rows[1]['__is_new']  = true;
            $nrseqhist = $this->newCode->getCodeByTableName('NRSEQHISTH', $this->environment->getNrOrgTrab(), 1, 4);
            $rows[1]['NRSEQHIST'] = $nrseqhist;
            $this->dataSourceName = 'paramempreinfh';

            $dtIniBackup = DateUtil::getDatadeString($rows[1]['DTINIVALIDADE'],
                $this->getFormat($rows[1]['DTINIVALIDADE']), true);

            $dtIni = DateUtil::getDatadeString($rows[0]['DTINIVALIDADE'],
                $this->getFormat($rows[0]['DTINIVALIDADE']), true);

            $dtFinBackup = $rows[1]['DTFINVALIDADE'] == null ? null :
                DateUtil::getDatadeString($rows[1]['DTFINVALIDADE'],
                    $this->getFormat($rows[1]['DTFINVALIDADE']), true);

            $dtFin = $rows[0]['DTFINVALIDADE'] == null ? null :
                DateUtil::getDatadeString($rows[0]['DTFINVALIDADE'],
                    $this->getFormat($rows[0]['DTFINVALIDADE']), true);

            // Compare dates is equals (3 to equal)
            if ((($dtFinBackup == null && $dtFin != null)
                  || ($dtFinBackup != null && $dtFin == null))
                || ( ($dtFinBackup != null && $dtFin != null)
                    && !DateUtil::comparaDatas($dtFinBackup, $dtFin, 3))) {
                $rows[1]['DTFINHISTORICO'] = $rows[1]['DTFINVALIDADE'];
                $rows[1]['DTFINVALIDADE'] = $rows[0]['DTFINVALIDADE'];
            } else {
                $rows[1]['DTFINHISTORICO'] = $rows[0]['DTFINVALIDADE'];
            }

            // Compare dates is equals (3 to equal)
            if (!DateUtil::comparaDatas($dtIniBackup, $dtIni, 3)) {
                $rows[1]['DTINIHISTORICO'] = $rows[1]['DTINIVALIDADE'];
                $rows[1]['DTINIVALIDADE'] = $rows[0]['DTINIVALIDADE'];
            } else {
                $rows[1]['DTINIHISTORICO'] = $rows[0]['DTINIVALIDADE'];
            }

            $this->crudService->save($this->dataSourceName, array($this->adicionaCamposInsercao($rows[1])));
            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function update(DTO\Request $request, DTO\Response $response){
        try {

           $ds = $request->getRow()['DSDIRCERTIFICADO'];
            $rows = $this->crudService->getRowsToSave($request, 'paramempreinf');
          if(count($ds) == 0){
              $request->getRow()['DSDIRCERTIFICADO'] = '';
          }
          $dataSet = $this->criaDataSet(array($request->getRow()));
          $request2 = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());

          $this->dataSourceNameToSave = 'paramempreinf';

           if(isset($rows[0]['CDSENHACERTDIG']) && ($rows[0]['senhaAlterada'] === true || $rows[0]['__is_new'] === true)) {
                $rows[0]['CDSENHACERTDIG'] = Utilities::zeedhiEncrypt($rows[0]['CDSENHACERTDIG']);
            }

            if($request->getRow()['DSDIRCERTIFICADO'] == ''){
            }else{
                $this->gravaCertificado($request->getRow());
            }

        } catch (\Exception $e) {
          $response->setError(new Error("Erro ao salvar certificado: ". $e->getMessage(), 500));
        }

    }

    const GET_INSCRICAO = "SELECT * FROM PARAMEMPREINF
                            WHERE CDEMPRESA = :CDEMPRESA
                            AND NRORG = :NRORG";

    public function consultaInscricao($row){
        try {
            $params = array(
                'CDEMPRESA' => $row[0]['CDEMPRESA'],
                'NRORG' => $this->environment->getNrOrgTrab()
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_INSCRICAO, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public  function criaDataSet($row){
        if($row[0]['__is_new']){
            if($row[0]['CDSENHACERTDIG']){
                $rows = $this->criptografarSenhaCertificado($row);
                $dataSet = new DataSet('paramempreinf', $rows);
            }else{
                $dataSet = new DataSet('paramempreinf', $row);
            }
            return $dataSet;
        }else{

            $resultadoBanco = $this->consultaInscricao($row);
            $inscricao = $resultadoBanco;

            if($row[0]['CDSENHACERTDIG'] !== $inscricao[0]['CDSENHACERTDIG']){
                if($row[0]['CDSENHACERTDIG']){
                    $rows = $this->criptografarSenhaCertificado($row);
                    $dataSet = new DataSet('paramempreinf', $rows);
                }else{
                    $dataSet = new DataSet('paramempreinf', $row);
                }
            }else{
                $dataSet = new DataSet('paramempreinf', $row);
            }
            return $dataSet;
        }
    }

    private function adicionaCamposInsercao($row) {
        $row['DTINCLUSAO'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
        $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
        return $row;
    }

    private function montaB64Certificado($dsDirCertificado) {
        if($dsDirCertificado) {
            $fileBase = explode('>', $dsDirCertificado);
            $fileName = $fileBase[0];
            $fileName = str_replace('<', '', $fileName);
            $fileLink = $fileBase[1];
            $file = file_get_contents($this->fileServerBaseUrl.$fileLink);
            $b64File = base64_encode($file);

            $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'certificado'.uniqid().'.pfx';
            file_put_contents($tmpFile, $file);
            $file = new File($tmpFile);
            $certificado = file_get_contents($tmpFile);
            $certPfx = base64_encode($certificado);

            $certPfx = 'data:application/x-pkcs12;base64,'.$certPfx;

            $lastModify = filectime($tmpFile);

            $certFileB64 = array(
                "name" => $fileName,
                "lastModified" => $lastModify,
                "lastModifiedDate" => date("D,d M Y H:i:s", $lastModify)." GMT",
                "webkitRelativePath" => '',
                "type" => $file->getMimeType(),
                "size"  => filesize($tmpFile),
                "b64File" => $certPfx);

            return $certFileB64;
        }
    }

    public function decrypt($text, $salt = 'teknisa'){
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, "\0"),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    private function getFormat($stringDate){
        //10 is brazilian format, more is date hour brazilian format
        if (strlen($stringDate) === 10) {
            return DateUtil::FORMATO_BRASILEIRO;
        } else {
            return DateUtil::FORMATO_BRASILEIRO_DATAHORA;
        }
    }

    public function saveXmlInFileUplouder($arquivo) {

        if (isset($arquivo)) {

            $file = array(
                "name" => time().'.'.explode('/', $arquivo["type"])[1],
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"],
                "b64File" => $arquivo["b64File"]);

            $row['DSDIRCERTIFICADO']    = array();
            $row['DSDIRCERTIFICADO'][0] = $file;

            $this->fileUploader->uploadFilesInRow($row, 'DSDIRCERTIFICADO', 'DSDIRCERTIFICADO');
          
            return $row['DSDIRCERTIFICADO'][0]['path'];

            /*

            $row = [
                'FILE' => [
                    [
                        "name" => time().'.'.explode('/', $arquivo["type"])[1],
                        "type" => $arquivo["type"],
                        "size" => $arquivo["size"],
                        "path" => '',
                        "b64File" => $arquivo["b64File"]
                    ]
                ],
                'FOLDER' => 'ANEXOLANCBOLREC'
            ];

            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */

        } else {
            return false;
        }
    }

    public function salvaAnexoGrid($row){

        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //$row['DSDIRCERTIFICADO'] = $this->geraNovoCodigo($row['NRORG'], 'DSDIRCERTIFICADO');
            //Salva o arquivo sempre como txt independente do seu formato

            $row['DSINFOANEXO'] = $row[0]["lastModified"] . '_' . $row[0]["lastModifiedDate"] . '_' .
                                $row[0]["webkitRelativePath"]. '_' . $row[0]["type"] . '_' .
                                $row[0]["size"] . '_' . $row[0]["name"];

            $row['NMPATHANEXO'] = $this->saveXmlInFileUplouder($row[0]);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraNovoCodigo($nrorg, $cdcontador, $intervalo = 1, $tamanho = 10){
        try {
            // Gera Novo Código Boleto
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg, $intervalo, $tamanho);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravaCertificado ($rowCertificado){

        try {
            $certificados = $rowCertificado['DSDIRCERTIFICADO'];
            $empresa = $rowCertificado['CDEMPRESA'];
            $nrorg = isset($rowCertificado['NRORG']) ? $rowCertificado['NRORG'] : $this->environment->getNrorg();


            $dsDirCert    =  '';
            $arrayCert    =  array();
           
            foreach ($certificados as $file) {
                $fileName  = $file['name'];
                $extension = explode(".", $fileName);
                $extension = end($extension);
                
                if(($extension !== self::EXTENSION_PFX) && ($extension !== self::EXTENSION_P12)) {
                    throw new \Exception($fileName.': extensão do arquivo não aceita.');
                }
                else {
                    $dsDirCert.='<'.$file['name'].'>';
                    $dsDirCert.=$file["path"];
                    $rowCertificado['DSDIRCERTIFICADO'] = $dsDirCert;

                    $parametros = array(
                        $DSCERTIFICADO = $rowCertificado['DSDIRCERTIFICADO'],
                        $CDEMPRESA = $rowCertificado['CDEMPRESA'],
                        $CDSENHACERTDIG = $rowCertificado['CDSENHACERTDIG'],
                        $NRORG = $rowCertificado['NRORG']
                    );
                    $this->Fis75001ParametrosEfdReinfService->updateDsDirCert($parametros);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function criptografarSenhaCertificado($row) {
        try {
            $senhaCriptografada = $this->encrypt($row[0]['CDSENHACERTDIG'], 'teknisa');
            $row[0]['CDSENHACERTDIG'] = $senhaCriptografada;
            return $row;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function encrypt($text, $salt = 'teknisa') {
        return Utilities::zeedhiEncrypt($text, $salt);
    }

}